<?php

/**
 * VtbpayPayment Class
 *
 * This class provides methods to handle payment operations with VTB API.
 */
class vtbpayPayment extends waPayment implements waIPayment
{
    private $order_id;
    
    
    /**
     * Get VTB API instance
     *
     * This method initializes and returns an instance of the VTB API using 
     * the provided configuration values.
     *
     * @return paymentVtbpayVtbApi An instance of the paymentVtbpayVtbApi
     */
    private function getVtbApi() 
    {
        return new paymentVtbpayVtbApi(
            $this->client_id,
            $this->client_secret,
            $this->test_mode,
            $this->merchant_authorization
        );
    }
    

    /**
     * Payment Method
     *
     * This method manages the payment process. It creates a new order, validates it,
     * retrieves the payment URL from VTB API, and then returns a view with the form URL.
     *
     * @param array $payment_form_data The form data for the payment
     * @param array $order_data The order data
     * @param bool $auto_submit Whether to auto-submit the form
     *
     * @return string The rendered view with the payment form
     *
     * @throws waException If the order currency is not supported
     */
    public function payment($payment_form_data, $order_data, $auto_submit = false)
    {
        $order = waOrder::factory($order_data);
        if (!in_array($order->currency_id, $this->allowedCurrency())) {
            throw new waException($this->_w('Payment error. Currency not supported.'));
        }
        
        // Return url
        $params = base64_encode(json_encode([
          'app_id' => $this->app_id, 
          'merchant_id' => $this->merchant_id
        ]));
        $return_url = $this->getRelayUrl() . '?params=' . $params;
        
        // Customer data
        if(
            !isset($order_data['customer_info']['email']) || 
            empty($email = $order_data['customer_info']['email'])
        ) {
            $contact = new waContact($order_data['contact_id']);
            $email = $contact->get('email', 'default');
        }
        
        // total
        $total = $order->tax_included == false && $order->tax ? $order->tax + $order->total : $order->total;
        
        $vtbpay = $this->getVtbApi();
        $response = $vtbpay->getOrderLink(
            $order->id, 
            $email, 
            time(), 
            $total, 
            $return_url
        );
        
        // Logging
        $this->setLog([
          'method_name' => 'payment',
          'request_method_name' => 'getOrderLink',
          'arguments' => [
            'order_id' => $order->id, 
            'email' => $email, 
            'time' => time(), 
            'total' => $total, 
            'return_url' => $return_url
          ],
          'response' => $response
        ]);
        
        if (
              isset($response['object']['payUrl']) && 
              !empty($response['object']['payUrl'])
        ) {
            $form_url = $response['object']['payUrl'];
        } 
        else {
            throw new waPaymentException($this->_w('Error: VTB payment system URL is unavailable.'));
        }

        $view = wa()->getView();
        $view->assign('form_url', $form_url);
        $view->assign('auto_submit', $auto_submit);
        $view->assign('form_desc', $this->_w('Redirecting to the bank website for payment...')); 
        $view->assign('form_btn', $this->_w('Go to the payment form, to the bank page'));
        return $view->fetch($this->path . '/templates/payment.html');
    }
    
    
    /**
     * Callback Init Method
     *
     * This method initializes the callback by extracting parameters from the request,
     * and then calls the parent callback method for further processing.
     *
     * @param array $request The request parameters
     *
     * @return array Parent's callback method result
     *
     * @throws waPaymentException If the invoice number is invalid
     */
    protected function callbackInit($request)
    {
        if (
              isset($request['orderId']) && !empty($request['orderId']) && 
              isset($request['params']) && !empty($request['params'])
        ) {
            $params = json_decode(base64_decode($request['params']), true);
            
            if (
                  isset($params['app_id']) && !empty($params['app_id']) && 
                  isset($params['merchant_id']) && !empty($params['merchant_id'])
            ) {
                $this->app_id = $params['app_id'];
                $this->merchant_id = $params['merchant_id'];
            }
            
            $this->order_id = $request['orderId'];
        } 
        elseif (
          isset($request['app_id']) && 
          !empty($request['app_id'])
        ) {
            $this->app_id = $request['app_id'];
        } 
        else {
            throw new waPaymentException($this->_w('Invalid invoice number.'));
        }
        // calling parent's method to continue plugin initialization
        return parent::callbackInit($request);
    }


    /**
     * Callback Handler Method
     *
     * This method manages the callback process. It retrieves the order data based on the order_id,
     * processes the transaction, and then redirects the customer based on the transaction status.
     *
     * @param array $request The request parameters
     *
     * @throws waPaymentException If the order ID or the order itself is not found
     */
    protected function callbackHandler($request)
    {
        if (!$this->order_id) {
            throw new waPaymentException($this->_w('Error: Order ID not found.'));
        }
        
        $order = $this->getAdapter()->getOrderData($this->order_id, $this);
        if (!$order) {
            throw new waPaymentException($this->_w('Error: Order not found.'));
        }
        
        $vtbpay = $this->getVtbApi();
        $response = $vtbpay->getOrderInfo($this->order_id);
        
        // Logging
        $this->setLog([
          'method_name' => 'callbackHandler',
          'request_method_name' => 'getOrderInfo',
          'arguments' => [
            'order_id' => $this->order_id
          ],
          'response' => $response
        ]);
        
        
        $transaction_data = $this->formalizeData($response);
        
        if (
              isset($response['object']['status']['value']) && 
              $response['object']['status']['value'] == 'PAID'
        ) {
            $app_payment_method = self::CALLBACK_PAYMENT;
            $transaction_data['state'] = self::STATE_CAPTURED;
            $transaction_data['type'] = self::OPERATION_AUTH_CAPTURE;
            $url = $this->getAdapter()->getBackUrl(waAppPayment::URL_SUCCESS, $transaction_data);
        } 
        else {
            $app_payment_method = self::CALLBACK_DECLINE;
            $transaction_data['state'] = self::STATE_DECLINED;
            $transaction_data['type'] = self::OPERATION_CANCEL;
            $url = $this->getAdapter()->getBackUrl(waAppPayment::URL_FAIL, $transaction_data);
        }

        $transaction_data = $this->saveTransaction($transaction_data, $request);
        $result = $this->execAppCallback($app_payment_method, $transaction_data);

        // Redirect customer
        wa()->getResponse()->redirect($url);
    }
    
    
    /**
     * Formalize Data Method
     *
     * This method formats the raw transaction data into a more usable format.
     *
     * @param array $transaction_raw_data The raw transaction data
     *
     * @return array The formatted transaction data
     */
    protected function formalizeData($transaction_raw_data)
    {
        $transaction_data = parent::formalizeData($transaction_raw_data);
        $transaction_data['order_id'] = ifset($transaction_raw_data['object']['orderId']);
        $transaction_data['amount'] = ifset($transaction_raw_data['object']['amount']['value']);
        $transaction_data['currency_id'] = ifset($transaction_raw_data['object']['amount']['code']);
        $transaction_data['native_id'] = $this->order_id;
        return $transaction_data;
    }


    /**
     * Allowed Currency Method
     *
     * This method returns the list of allowed currencies for the payment.
     *
     * @return array List of allowed currencies
     */
    public function allowedCurrency() 
    {
        return [
          'RUB'
        ];
    }
    
    
    /**
     * Log Method
     *
     * This method logs the provided data if logging is enabled.
     *
     * @param array $data The data to be logged
     */
    private function setLog($data) 
    {
      if ($this->logging) self::log($this->id, $data);
    }
}
